# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateNLevXCindex(Template):
    'RP note: not sure what this does yet!'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal', 'Correlated classifications' ]
    engines = ['eStat']

    inputs = '''
y1 = DataVector('Response 1: ')
y2 = DataVector('Response 2: ')
CC1a = IDVector('First Correlated Classification for response 1: ')
CC1b = IDVector('First Correlated Classification for response 2: ')
CC2a = IDVector('Second Correlated Classification for response 1: ')
CC2b = IDVector('Second Correlated Classification for response 2: ')

x = DataMatrix('Explanatory variables: ', allow_cat = True)

storeresid = Boolean('Store residuals?')

beta = ParamVector(parents=[x], as_scalar=True)

omega = ParamMatrix(modelled = False, customstep=True)
omega.size = 2
d = ParamMatrix(customstep=True)
d.size = 2
priors = Text('Priors: ', ['Uniform', 'Wishart'])
if priors == 'Wishart':
    R = List('R matrix: ')
    v = Integer('Degrees of Freedom:')

omega_e = ParamMatrix(modelled = False, customstep=True)
omega_e.size = 2
d_e = ParamMatrix(customstep=True)
d_e.size = 2
priors_e = Text('Priors at lower level: ', ['Uniform', 'Wishart'])
if priors_e == 'Wishart':
    R_e = List('R matrix: ')
    v_e = Integer('Degrees of Freedom:')

if storeresid:
    v1 = ParamVector(parents=[CC1a], as_scalar=False)
    v2 = ParamVector(parents=[CC2a], as_scalar=False)
else:
    v1 = ParamVector(parents=[CC1a], as_scalar=False, monitor=False)
    v2 = ParamVector(parents=[CC2a], as_scalar=False, monitor=False)

deviance = ParamScalar(modelled = False)
'''

    model = '''
model {
    for (i in 1:length(${y1})) {
        dummyy[i] ~ dnormal2a(${y1}[i], ${y2}[i], mu1[i], mu2[i], d_e[0], d_e[1], d_e[2])
        mu1[i] <- ${mmult(x, 'beta', 'i')}
        + v1[${CC1a}[i]] + v2[${CC2a}[i]]
        mu2[i] <- ${mmult(x, 'beta', 'i')}
        + v1[${CC1b}[i]] + v2[${CC2b}[i]]
    }

    for (j in 1:length(v1)) {
        dummy[j] ~ dnormal2a(v1[j], v2[j], 0, 0, d[0], d[1], d[2])
        dum[j] ~ ddummy(dummy[j])
        v1[j] ~ dflat()
        v2[j] ~ dflat()
    }    

    # Priors
    % for i in range(0, x.ncols()):
    beta_${i} ~ dflat()
    % endfor
}    
'''

    preccode = '''
        <%!
    def mmult2(names, var, index,count):
        out = ""
        first = True
        for name in names:
            if first == False:
                out += ' + '
            else:
                first = False
            out += 'double(' + name + '[' + index + ']) * ' + var + '_' + str(count)
            count += 1
        return out
%>
    {
        // Note currently using a uniform prior for variance matrix
        SymMatrix sb(2);
        for(int i = 0; i < length(v1); i++) {
% for j in range(0, 2):
% for k in range(j, 2):
            sb(${j}, ${k}) += double(v${j + 1}[i]) * double(v${k + 1}[i]);
% endfor
% endfor
        }
% if priors == 'Uniform':
        int vw = length(v1)-3;
        if (runstate == 0) {
% for j in range(0, 2):
            sb(${j}, ${j}) += 0.0001;
% endfor
        }
% endif
% if priors == 'Wishart':
        int vw = length(v1) + ${v};
% endif

% if priors == 'Wishart':
<%
import numpy
Rmat = numpy.empty([2, 2])
count = 0 
for j in range(0, 2):
    for k in range(0, j + 1):
        Rmat[j, k] = float(R.name[count]) 
        Rmat[k, j] = Rmat[j, k]
        count += 1
%>

<% count = 0 %>
% for j in range(0, 2):
% for k in range(j, 2):
        sb(${j}, ${k}) += ${str(Rmat[j][k] * float(v))};
% endfor
% endfor
% endif

        matrix_sym_invinplace(sb);
        bool fail = false;
        mat_d = dwishart(vw, sb, fail);
        mat_omega = matrix_sym_inverse(mat_d, fail);

        // Note currently using a uniform prior for variance matrix
        SymMatrix sbe(2);
        for(int i = 0; i < length(${y1}); i++) {
            double e0 = double(${y1}[i]) - ${mmult2(x, 'beta', 'i', 0)} - double(v1[int(${CC1a}[i])]) - double(v2[int(${CC2a}[i])]);
            double e1 = double(${y2}[i]) - ${mmult2(x, 'beta', 'i', 0)} - double(v1[int(${CC1b}[i])]) - double(v2[int(${CC2b}[i])]);
% for j in range(0, 2):
% for k in range(j, 2):
            sbe(${j}, ${k}) += e${j} * e${k};
% endfor
% endfor
        }
% if priors == 'Uniform':
        int vwe = length(${y1})-3;
        if (runstate == 0) {
% for j in range(0, 2):
            sbe(${j}, ${j}) += 0.0001;
% endfor
        }
% endif
% if priors == 'Wishart':
        int vwe = length(${y1}) + ${v_e};
% endif
% if priors == 'Wishart':
<%
import numpy
Rmat = numpy.empty([2, 2])
count = 0 
for j in range(0, 2):
    for k in range(0, j + 1):
        Rmat[j, k] = float(R_e[count]) 
        Rmat[k, j] = Rmat[j, k]
        count += 1
%>

<% count = 0 %>
% for j in range(0, 2):
% for k in range(j, 2):
        sbe(${j}, ${k}) += ${str(Rmat[j, k] * float(v_e))};
% endfor
% endfor
% endif

        matrix_sym_invinplace(sbe);
        mat_d_e = dwishart(vwe, sbe, fail);
        mat_omega_e = matrix_sym_inverse(mat_d_e, fail);
    }
         '''
